/***********************************************************************
* IPCV - Scilab Image Processing and Computer Vision toolbox
* Original work Copyright (C) 2017  Tan Chin Luh
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
***********************************************************************/



#include "common.h"

int SetImage(int nPos, Mat& new_img,void* pvApiCtx)
{

	Size s = new_img.size();
	int rows = new_img.size().height;
	int cols = new_img.size().width;
	int ndims;
	if 	(new_img.channels() ==3 )
	{ ndims = new_img.channels();}
	else
	{ndims = 2;}

	//int ndims = new_img.ndims;
	int dims_var[3] = {rows,cols,ndims};
	int *dims;
	dims = dims_var;
	void * pMatData;


	SciErr sciErr;
	//sciprint("-----------------------\n");
	//sciprint("Set Image Properties...\n");
	//sciprint("-----------------------\n");
	//sciprint("dims : %i\n",new_img.dims);
	//sciprint("rows  : %i\n",new_img.rows );
	//sciprint("cols  : %i\n",new_img.cols );
	//sciprint("channels  : %i\n",new_img.channels() );
	//sciprint("dims  : %i\t %i\t %i\t\n",*dims, *(dims+1), *(dims+2));
	//sciprint("depth  : %i\n",new_img.depth());

	switch(new_img.depth())
	{
	case CV_8U:
		{

			//////////////////////////
			// 3D Uint8 Scilab Image
			//////////////////////////

			if (ndims==3)
			{
				//sciprint("CV_8U\n");
				pMatData = malloc(new_img.size().width * new_img.size().height * new_img.channels() * new_img.elemSize1());
				matdata2scidata(new_img, pMatData);

				sciErr = createHypermatOfUnsignedInteger8(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, dims, ndims, (const unsigned char*)pMatData);
				if (sciErr.iErr)
				{
					printError(&sciErr, 0);
					return sciErr.iErr;
				}

				AssignOutputVariable(pvApiCtx, nPos) = nbInputArgument(pvApiCtx) + nPos;
			}
			//////////////////////////
			// 2D Uint8 Scilab Image
			//////////////////////////

			else if (ndims == 2)
			{

				double s1 = sum(new_img==0)[0]/255;
				double s2 = sum(new_img==255)[0]/255;
				double s3 = new_img.rows*new_img.cols;

				if (s3 == (s1+s2))
				{
					//sciprint("binary");

					Mat new_img2;
					//new_img2 = Mat(2, sz2, CV_32S);

					new_img.convertTo(new_img2,CV_32S,1.0/255);

					pMatData = malloc(new_img2.size().width * new_img2.size().height * new_img2.channels() * new_img2.elemSize1());
					matdata2scidata(new_img2, pMatData);
					sciErr = createMatrixOfBoolean(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, rows, cols, (int*)pMatData);
					if(sciErr.iErr)
					{
						printError(&sciErr, 0);
						return sciErr.iErr;
					}
					AssignOutputVariable(pvApiCtx, nPos) = nbInputArgument(pvApiCtx) + nPos;
				}

				else
				{
					////
					pMatData = malloc(new_img.size().width * new_img.size().height * new_img.channels() * new_img.elemSize1());
					matdata2scidata(new_img, pMatData);
					sciErr = createMatrixOfUnsignedInteger8(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, rows, cols, (unsigned char*)pMatData);
					if(sciErr.iErr)
					{
						printError(&sciErr, 0);
						return sciErr.iErr;
					}
					AssignOutputVariable(pvApiCtx, nPos) = nbInputArgument(pvApiCtx) + nPos;
				}

				////

			}
			else
			{
				Scierror(999, ("%s: Wrong type for input argument: An Image expected.\n"), 1);
				return -1;
			}

		}
		break;
	case CV_8S:
		{
			//////////////////////////
			// 3D int8 Scilab Image
			//////////////////////////

			if (ndims==3)
			{
				//sciprint("CV_8U\n");
				pMatData = malloc(new_img.size().width * new_img.size().height * new_img.channels() * new_img.elemSize1());
				matdata2scidata(new_img, pMatData);

				sciErr = createHypermatOfInteger8(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, dims, ndims, (const char*)pMatData);
				if (sciErr.iErr)
				{
					printError(&sciErr, 0);
					return sciErr.iErr;
				}

				AssignOutputVariable(pvApiCtx, nPos) = nbInputArgument(pvApiCtx) + nPos;
			}
			//////////////////////////
			// 2D int8 Scilab Image
			//////////////////////////

			else if (ndims == 2)
			{
				pMatData = malloc(new_img.size().width * new_img.size().height * new_img.channels() * new_img.elemSize1());
				matdata2scidata(new_img, pMatData);

				//sciErr = createMatrixOfUnsignedInteger8(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, rows, cols, (const unsigned char*)new_img.data);
				sciErr = createMatrixOfInteger8(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, rows, cols, (char*)pMatData);
				if(sciErr.iErr)
				{
					printError(&sciErr, 0);
					return sciErr.iErr;
				}
				AssignOutputVariable(pvApiCtx, nPos) = nbInputArgument(pvApiCtx) + nPos;
			}
			else
			{
				Scierror(999, ("%s: Wrong type for input argument: An Image expected.\n"), 1);
				return -1;
			}

		}
		break;
	case CV_16U:
		{

			//////////////////////////
			// 3D Uint16 Scilab Image
			//////////////////////////

			if (ndims==3)
			{
				//sciprint("CV_8U\n");
				pMatData = malloc(new_img.size().width * new_img.size().height * new_img.channels() * new_img.elemSize1());
				matdata2scidata(new_img, pMatData);

				sciErr = createHypermatOfUnsignedInteger16(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, dims, ndims, (const unsigned short*)pMatData);
				if (sciErr.iErr)
				{
					printError(&sciErr, 0);
					return sciErr.iErr;
				}

				AssignOutputVariable(pvApiCtx, nPos) = nbInputArgument(pvApiCtx) + nPos;
			}
			//////////////////////////
			// 2D Uint16 Scilab Image
			//////////////////////////

			else if (ndims == 2)
			{
				pMatData = malloc(new_img.size().width * new_img.size().height * new_img.channels() * new_img.elemSize1());
				matdata2scidata(new_img, pMatData);

				//sciErr = createMatrixOfUnsignedInteger8(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, rows, cols, (const unsigned char*)new_img.data);
				sciErr = createMatrixOfUnsignedInteger16(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, rows, cols, (unsigned short*)pMatData);
				if(sciErr.iErr)
				{
					printError(&sciErr, 0);
					return sciErr.iErr;
				}
				AssignOutputVariable(pvApiCtx, nPos) = nbInputArgument(pvApiCtx) + nPos;
			}
			else
			{
				Scierror(999, ("%s: Wrong type for input argument: An Image expected.\n"), 1);
				return -1;
			}

		}
		break;
	case CV_16S:
		{

			//////////////////////////
			// 3D int16 Scilab Image
			//////////////////////////

			if (ndims==3)
			{
				//sciprint("CV_8U\n");
				pMatData = malloc(new_img.size().width * new_img.size().height * new_img.channels() * new_img.elemSize1());
				matdata2scidata(new_img, pMatData);

				sciErr = createHypermatOfInteger16(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, dims, ndims, (const short*)pMatData);
				if (sciErr.iErr)
				{
					printError(&sciErr, 0);
					return sciErr.iErr;
				}

				AssignOutputVariable(pvApiCtx, nPos) = nbInputArgument(pvApiCtx) + nPos;
			}
			//////////////////////////
			// 2D int16 Scilab Image
			//////////////////////////

			else if (ndims == 2)
			{
				pMatData = malloc(new_img.size().width * new_img.size().height * new_img.channels() * new_img.elemSize1());
				matdata2scidata(new_img, pMatData);

				//sciErr = createMatrixOfUnsignedInteger8(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, rows, cols, (const unsigned char*)new_img.data);
				sciErr = createMatrixOfInteger16(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, rows, cols, (short*)pMatData);
				if(sciErr.iErr)
				{
					printError(&sciErr, 0);
					return sciErr.iErr;
				}
				AssignOutputVariable(pvApiCtx, nPos) = nbInputArgument(pvApiCtx) + nPos;
			}
			else
			{
				Scierror(999, ("%s: Wrong type for input argument: An Image expected.\n"), 1);
				return -1;
			}

		}
		break;
	case CV_32S:
		{

			//////////////////////////
			// 3D int32 Scilab Image
			//////////////////////////

			if (ndims==3)
			{
				//sciprint("CV_8U\n");
				pMatData = malloc(new_img.size().width * new_img.size().height * new_img.channels() * new_img.elemSize1());
				matdata2scidata(new_img, pMatData);

				sciErr = createHypermatOfInteger32(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, dims, ndims, (const int*)pMatData);
				if (sciErr.iErr)
				{
					printError(&sciErr, 0);
					return sciErr.iErr;
				}

				AssignOutputVariable(pvApiCtx, nPos) = nbInputArgument(pvApiCtx) + nPos;
			}
			//////////////////////////
			// 2D int32 Scilab Image
			//////////////////////////

			else if (ndims == 2)
			{
				pMatData = malloc(new_img.size().width * new_img.size().height * new_img.channels() * new_img.elemSize1());
				matdata2scidata(new_img, pMatData);

				//sciErr = createMatrixOfUnsignedInteger8(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, rows, cols, (const unsigned char*)new_img.data);
				sciErr = createMatrixOfInteger32(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, rows, cols, (int*)pMatData);
				if(sciErr.iErr)
				{
					printError(&sciErr, 0);
					return sciErr.iErr;
				}
				AssignOutputVariable(pvApiCtx, nPos) = nbInputArgument(pvApiCtx) + nPos;
			}
			else
			{
				Scierror(999, ("%s: Wrong type for input argument: An Image expected.\n"), 1);
				return -1;
			}

		}
		break;


	case CV_32F:
		//sciprint("CV_32F\n");
		{
			//sciprint("CV_64F\n");
			new_img.convertTo(new_img,CV_64F);

			if (ndims == 3)
			{
				pMatData = malloc(new_img.size().width * new_img.size().height * new_img.channels() * new_img.elemSize1());
				matdata2scidata(new_img, pMatData);

				sciErr = createHypermatOfDouble(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, dims, ndims, (double*)pMatData);
				if (sciErr.iErr)
				{
					printError(&sciErr, 0);
					return sciErr.iErr;
				}

				AssignOutputVariable(pvApiCtx, nPos) = nbInputArgument(pvApiCtx) + nPos;
			}
			else if (ndims == 2 )
			{

				pMatData = malloc(new_img.size().width * new_img.size().height * new_img.channels() * new_img.elemSize1());
				matdata2scidata(new_img, pMatData);

				sciErr = createMatrixOfDouble(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, rows, cols, (double*)pMatData);
				if(sciErr.iErr)
				{
					printError(&sciErr, 0);
					return sciErr.iErr;
				}
				AssignOutputVariable(pvApiCtx, nPos) = nbInputArgument(pvApiCtx) + nPos;

			}
			else
			{
				Scierror(999, ("%s: Wrong type for input argument: An Image expected.\n"), 1);
				return -1;
			}
		}
		break;
	case CV_64F:
		{
			//sciprint("CV_64F\n");

			if (ndims == 3)
			{
				pMatData = malloc(new_img.size().width * new_img.size().height * new_img.channels() * new_img.elemSize1());
				matdata2scidata(new_img, pMatData);

				sciErr = createHypermatOfDouble(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, dims, ndims, (double*)pMatData);
				if (sciErr.iErr)
				{
					printError(&sciErr, 0);
					return sciErr.iErr;
				}

				AssignOutputVariable(pvApiCtx, nPos) = nbInputArgument(pvApiCtx) + nPos;
			}
			else if (ndims == 2 )
			{

				pMatData = malloc(new_img.size().width * new_img.size().height * new_img.channels() * new_img.elemSize1());
				matdata2scidata(new_img, pMatData);

				sciErr = createMatrixOfDouble(pvApiCtx, nbInputArgument(pvApiCtx) + nPos, rows, cols, (double*)pMatData);
				if(sciErr.iErr)
				{
					printError(&sciErr, 0);
					return sciErr.iErr;
				}
				AssignOutputVariable(pvApiCtx, nPos) = nbInputArgument(pvApiCtx) + nPos;

			}
			else
			{
				Scierror(999, ("%s: Wrong type for input argument: An Image expected.\n"), 1);
				return -1;
			}
		}
		break;
	default:
		sciprint("Unknown type !\n"); // Should never happen
	}
	//sciErr = createHypermatOfUnsignedInteger8(pvApiCtx, nbInputArgument(pvApiCtx) + 1, dims, ndims, (const unsigned char*)new_img.data);
	free(pMatData);
	//free(pSrc);
	//free(pDst);
	return 0;
}

//int scidata2matdata(Mat &pImage, void* pMatData)
int matdata2scidata(Mat &pImage, void *pMatData)
{
	//  IPL_DEPTH_8U, IPL_DEPTH_8S, IPL_DEPTH_16U,
	//IPL_DEPTH_16S, IPL_DEPTH_32S, IPL_DEPTH_32F and IPL_DEPTH_64F
	//int row, col, ch;
	int rows, cols;
	long nCount = 0;
	int nBytes;

	unsigned char * pSrc = NULL;
	unsigned char * pDst = NULL;

	//if (pImage == NULL || pMatData == NULL)
	//	return FALSE;

	/////////////
	//pSrc = (unsigned char*)(pImage.data);
	//pDst = (unsigned char*)pMatData;

	///*how many bytes per pixel per channel*/
	//nBytes = pImage.elemSize1();
	//
	//for(ch = 0; ch < pImage.channels() ; ch++) //the order of IplImage is BGR
	//	for(col =0; col < pImage.size().width; col++)
	//		for(row = 0; row < pImage.size().height; row++)
	//		{
	//			memcpy(pDst+nCount, pSrc + pImage.step*row + (col*pImage.channels() + (pImage.channels()-ch-1))*nBytes, nBytes );
	//			nCount += nBytes;
	//			//sciprint(".");
	//		}
	/////////////

	///////////////////////////////////////
	pImage = pImage.t();
	pSrc = (unsigned char*)(pImage.data);
	pDst = (unsigned char*)pMatData;
	rows = pImage.rows;
	cols = pImage.cols;
	vector<Mat> ch1; // B, G, R channels
	split(pImage, ch1);
	nBytes = pImage.elemSize1();	


	//unsigned char * data = NULL;
	//data = (unsigned char*)pMatData;

	if (pImage.channels()==1)
	{
		memcpy(pDst, ch1[0].ptr(), rows*cols*nBytes);
	}
	else
	{
		memcpy(pDst, ch1[2].ptr(), rows*cols*nBytes);
		memcpy(pDst+rows*cols*nBytes, ch1[1].ptr(), rows*cols*nBytes);
		memcpy(pDst+2*rows*cols*nBytes, ch1[0].ptr(), rows*cols*nBytes);
	}

	///////////////////////////////////////



	//memcpy(pDst + pImage.step*row + (col*pImage.channels() + (pImage.channels()-ch-1))*nBytes, pSrc+nCount, nBytes );
	//sciprint("Test3\n");

	//pDst = (unsigned char*)pImage.data;
	//	pSrc = (unsigned char*)pMatData;
	//
	//	
	//
	//	/*how many bytes per pixel per channel*/
	//	nBytes = pImage.elemSize1();
	//
	//	for(ch = 0; ch < pImage.channels() ; ch++) //the order of IplImage is BGR
	//		for(col =0; col < pImage.size().width; col++)
	//			for(row = 0; row < pImage.size().height; row++)
	//			{
	//				memcpy(pDst + pImage.step*row + (col*pImage.channels() + (pImage.channels()-ch-1))*nBytes, pSrc+nCount, nBytes );
	//				nCount += nBytes;
	//			}
	//free(pMatData);
	//free(pSrc);
	//free(pDst);
	return 0;
}

//int scidata2matdata(Mat &pImage, unsigned char* pMatData)
int scidata2matdata(Mat &pImage, void* pMatData)
{
	//int row, col, ch;
	int rows, cols;
	long nCount = 0;
	int nBytes;

	unsigned char * pDst = NULL;
	unsigned char * pSrc = NULL;
	//sciprint("pucData : %d \t %d \t %c \t %c \t\n", pMatData, &pMatData, pMatData, &pMatData);
	//if (pImage == NULL || pMatData == NULL)
	//	return FALSE;
	//sciprint("Check Point 1\n");


	////////////////
	//pDst = (unsigned char*)pImage.data;
	//pSrc = (unsigned char*)pMatData;
	///*how many bytes per pixel per channel*/
	//nBytes = pImage.elemSize1();
	//for(ch = 0; ch < pImage.channels() ; ch++) //the order of IplImage is BGR
	//	for(col =0; col < pImage.size().width; col++)
	//		for(row = 0; row < pImage.size().height; row++)
	//		{
	//			memcpy(pDst + pImage.step*row + (col*pImage.channels() + (pImage.channels()-ch-1))*nBytes, pSrc+nCount, nBytes );
	//			nCount += nBytes;
	//		}
	//////////////////

	///////////////////////////////////////
	//pImage = pImage.t();
	pImage = pImage.t();
	pDst = (unsigned char*)(pImage.data);
	pSrc = (unsigned char*)pMatData;
	rows = pImage.rows;
	cols = pImage.cols;

	nBytes = pImage.elemSize1();	
	//nBytes = 1;	
	//sciprint("nBytes : %i\n",nBytes);
	if (pImage.channels()==1)
	{
		memcpy(pDst, pSrc, rows*cols*nBytes);
	}
	else
	{
		vector<Mat> ch1; // B, G, R channels
		split(pImage, ch1);

		//memcpy(pDst, ch1[2].ptr(), rows*cols*nBytes);
		//memcpy(pDst+rows*cols, ch1[1].ptr(), rows*cols*nBytes);
		//memcpy(pDst+2*rows*cols, ch1[0].ptr(), rows*cols*nBytes);
		memcpy(ch1[2].ptr(), pSrc, rows*cols*nBytes);
		memcpy(ch1[1].ptr(), pSrc+rows*cols*nBytes, rows*cols*nBytes);
		memcpy(ch1[0].ptr(), pSrc+2*rows*cols*nBytes, rows*cols*nBytes);
		//ch1[2].data = pSrc;
		//ch1[1].data = pSrc+rows*cols;
		//ch1[0].data = pSrc+2*rows*cols;
		merge(ch1,pImage);
	}

	pImage = pImage.t();

	//////////////////////
	//int ch,col;
	//sciprint("This line show pDst\n");
	//for(ch = 0; ch < rows ; ch++) 
	//{
	//	for(col =0; col < cols; col++)		
	//	{
	//		//sciprint("%i\t%i\t%i\n",&pDst,pDst,*pDst);
	//		//*pDst++;
	//		sciprint("%d\t",*(pDst+ch*pImage.step+col)); //
	//		//sciprint("%f\t",pDst[ch*pImage.step+col]);
	//	}
	//	sciprint("\n");
	//}
	//////////////////////
	return 0;
}


int GetImage(int nPos, Mat& new_img,void* pvApiCtx)
{

	SciErr sciErr;
	int* piAddr = NULL;
	int iType   = 0;
	int iRet    = 0;
	int * dims = NULL;
	int ndims;
	double* pdblReal	= NULL;

	sciErr = getVarAddressFromPosition(pvApiCtx, nPos, &piAddr);
	if (sciErr.iErr)
	{
		printError(&sciErr, 0);
		return -1;
	}

	sciErr = getVarType(pvApiCtx, piAddr, &iType);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return -1;
	}

	//sciprint("\tType: %i : ", iType);

	switch(iType)
	{
	case sci_matrix:
		{
			if (isHypermatType(pvApiCtx, piAddr))
			{
				/////////////////////
				// 3D Double Image //
				/////////////////////
				sciErr = getHypermatOfDouble(pvApiCtx, piAddr, &dims, &ndims, &pdblReal);
				if (sciErr.iErr)
				{
					printError(&sciErr, 0);
					return -1;
				}
				int sz2[] = {*dims, *(dims+1), *(dims+2)};
				//new_img = Mat(ndims, sz2, CV_64F, pdblReal);	
				new_img = Mat(2, sz2, CV_64FC3);
				scidata2matdata(new_img, pdblReal);
			}
			else
			{

				//sciprint("2D Double Image Detected...\n");
				int iRows			= 0;
				int iCols			= 0;

				/////////////////////
				// 2D Double Image //
				/////////////////////

				sciErr = getMatrixOfDouble(pvApiCtx, piAddr, &iRows, &iCols, &pdblReal);
				if(sciErr.iErr)
				{
					printError(&sciErr, 0);
					return sciErr.iErr;
				}

				int sz2[] = {iRows,iCols};

				new_img = Mat(2, sz2, CV_64F);
				scidata2matdata(new_img, pdblReal);
			}


		}
		break;
	case sci_poly:
		sciprint("A matrix of polynomials\n");
		break;
	case sci_boolean:
		{
			if (isHypermatType(pvApiCtx, piAddr))
			{
				int* piBool	= NULL;
				/////////////////////
				// 3D Boolean Image //
				/////////////////////
				sciErr = getHypermatOfBoolean(pvApiCtx, piAddr, &dims, &ndims, &piBool);
				//sciErr = getHypermatOfInteger32(pvApiCtx, piAddr, &dims, &ndims, &piBool);			
				if (sciErr.iErr)
				{
					printError(&sciErr, 0);
					return -1;
				}
				int sz2[] = {*dims, *(dims+1), *(dims+2)};
				//new_img = Mat(ndims, sz2, CV_64F, pdblReal);	
				//new_img = Mat(2, sz2, CV_64FC3);
				//scidata2matdata(new_img, pdblReal);

				/////////////////////
				//int sz2[] = {iRows,iCols};

				Mat new_img2;
				new_img2 = Mat(2, sz2, CV_32SC3);
				scidata2matdata(new_img2, piBool);
				new_img = Mat(2, sz2, CV_8UC3);
				new_img2.convertTo(new_img,CV_8UC3,255);
			}
			else
			{

				int iRows	= 0;
				int iCols	= 0;
				int* piBool	= NULL;
				sciErr = getMatrixOfBoolean(pvApiCtx, piAddr, &iRows, &iCols, &piBool);
				if(sciErr.iErr)
				{
					printError(&sciErr, 0);
					return -1;
				}
				int sz2[] = {iRows,iCols};

				Mat new_img2;
				new_img2 = Mat(2, sz2, CV_32S);
				scidata2matdata(new_img2, piBool);

				new_img = Mat(2, sz2, CV_8U);
				new_img2.convertTo(new_img,CV_8U,255);
			}

		}
		break;
	case sci_sparse:
		sciprint("A sparse matrix of doubles\n");
		break;
	case sci_boolean_sparse:
		sciprint("A sparse matrix of booleans\n");
		break;
	case sci_matlab_sparse:
		sciprint("A sparse matlab matrix\n");
		break;
	case sci_ints:
		//sciprint("A matrix of integers\n");
		{
			int precision;
			sciErr = getMatrixOfIntegerPrecision(pvApiCtx, piAddr, &precision);
			if (sciErr.iErr)
			{
				printError(&sciErr, 0);
				return sciErr.iErr;
			}
			//sciprint("datatype found: %i\n",precision);
			switch (precision)
			{
			case SCI_INT8:
				{
					if (isHypermatType(pvApiCtx, piAddr))
					{
						//void * data = NULL;
						char* pucData  = NULL;

						//sciprint("Precision : %i - SCI_UINT8\n", precision);
						/////////////////////
						// 3D INT8 Image //
						/////////////////////
						//sciErr = getHypermatOfUnsignedInteger8(pvApiCtx, piAddr, &dims, &ndims, (unsigned char**)&data);
						sciErr = getHypermatOfInteger8(pvApiCtx, piAddr, &dims, &ndims, &pucData);
						if (sciErr.iErr)
						{
							printError(&sciErr, 0);
							return -1;
						}
						int sz2[] = {*dims, *(dims+1), *(dims+2)};

						new_img = Mat(2, sz2, CV_8SC3);
						scidata2matdata(new_img, pucData);
					}
					else
					{

						int iRows               = 0;
						int iCols               = 0;
						char* pucData  = NULL;

						//sciprint("Precision : %i - SCI_UINT8\n", precision);
						/////////////////////
						// 2D UINT8 Image //
						/////////////////////
						sciErr = getMatrixOfInteger8(pvApiCtx, piAddr, &iRows, &iCols, &pucData);
						if (sciErr.iErr)
						{
							printError(&sciErr, 0);
							return sciErr.iErr;
						}
						int sz2[] = {iRows,iCols};

						new_img = Mat(2, sz2, CV_8S);
						scidata2matdata(new_img, pucData);
					}
				}
				break;
			case SCI_UINT8:
				{

					if (isHypermatType(pvApiCtx, piAddr))
					{
						//void * data = NULL;
						unsigned char* pucData  = NULL;

						//sciprint("Precision : %i - SCI_UINT8\n", precision);
						/////////////////////
						// 3D UINT8 Image //
						/////////////////////
						sciErr = getHypermatOfUnsignedInteger8(pvApiCtx, piAddr, &dims, &ndims, &pucData);
						if (sciErr.iErr)
						{
							printError(&sciErr, 0);
							return -1;
						}
						int sz2[] = {*dims, *(dims+1), *(dims+2)};
						new_img = Mat(2, sz2, CV_8UC3);
						scidata2matdata(new_img, pucData);
					}
					else
					{
						int iRows               = 0;
						int iCols               = 0;
						unsigned char* pucData  = NULL;

						/////////////////////
						// 2D UINT8 Image //
						/////////////////////
						sciErr = getMatrixOfUnsignedInteger8(pvApiCtx, piAddr, &iRows, &iCols, &pucData);
						if (sciErr.iErr)
						{
							printError(&sciErr, 0);
							return sciErr.iErr;
						}
						int sz2[] = {iRows,iCols};

						new_img = Mat(2, sz2, CV_8U);
						scidata2matdata(new_img, pucData);
					}

				}
				break;
			case SCI_INT16:
				{
					if (isHypermatType(pvApiCtx, piAddr))
					{
						//void * data = NULL;
						short* pucData  = NULL;

						//sciprint("Precision : %i - SCI_UINT16\n", precision);
						/////////////////////
						// 3D INT16 Image //
						/////////////////////
						//sciErr = getHypermatOfUnsignedInteger8(pvApiCtx, piAddr, &dims, &ndims, (unsigned char**)&data);
						sciErr = getHypermatOfInteger16(pvApiCtx, piAddr, &dims, &ndims, &pucData);
						if (sciErr.iErr)
						{
							printError(&sciErr, 0);
							return -1;
						}
						int sz2[] = {*dims, *(dims+1), *(dims+2)};
						//new_img = Mat(2, sz2, CV_8UC3, pucData);	

						//int sz2[] = {iCols,iRows};

						new_img = Mat(2, sz2, CV_16SC3);
						scidata2matdata(new_img, pucData);
					}
					else
					{

						int iRows               = 0;
						int iCols               = 0;
						short* pucData  = NULL;

						/////////////////////
						// 2D UINT16 Image //
						/////////////////////
						sciErr = getMatrixOfInteger16(pvApiCtx, piAddr, &iRows, &iCols, &pucData);
						if (sciErr.iErr)
						{
							printError(&sciErr, 0);
							return sciErr.iErr;
						}
						int sz2[] = {iRows,iCols};

						new_img = Mat(2, sz2, CV_16S);
						scidata2matdata(new_img, pucData);
					}

				}
				break;

			case SCI_UINT16:
				{
					if (isHypermatType(pvApiCtx, piAddr))
					{
						unsigned short* pucData  = NULL;

						//sciprint("Precision : %i - SCI_UINT16\n", precision);
						/////////////////////
						// 3D UINT16 Image //
						/////////////////////
						//sciErr = getHypermatOfUnsignedInteger8(pvApiCtx, piAddr, &dims, &ndims, (unsigned char**)&data);
						sciErr = getHypermatOfUnsignedInteger16(pvApiCtx, piAddr, &dims, &ndims, &pucData);
						if (sciErr.iErr)
						{
							printError(&sciErr, 0);
							return -1;
						}
						int sz2[] = {*dims, *(dims+1), *(dims+2)};
						//new_img = Mat(2, sz2, CV_8UC3, pucData);	

						//int sz2[] = {iCols,iRows};

						new_img = Mat(2, sz2, CV_16UC3);
						scidata2matdata(new_img, pucData);
					}
					else
					{

						int iRows               = 0;
						int iCols               = 0;
						unsigned short* pucData  = NULL;

						/////////////////////
						// 2D UINT16 Image //
						/////////////////////
						sciErr = getMatrixOfUnsignedInteger16(pvApiCtx, piAddr, &iRows, &iCols, &pucData);
						if (sciErr.iErr)
						{
							printError(&sciErr, 0);
							return sciErr.iErr;
						}
						int sz2[] = {iRows,iCols};

						new_img = Mat(2, sz2, CV_16U);
						scidata2matdata(new_img, pucData);
					}

				}
				break;

			case SCI_INT32:
				{
					if (isHypermatType(pvApiCtx, piAddr))
					{
						//sciErr = getHypermatOfInteger32(pvApiCtx, piAddr, &dims, &ndims, (int*)&data);
						//if(sciErr.iErr)
						//{
						//	printError(&sciErr, 0);
						//	return sciErr.iErr;
						//}
						sciprint("Precision : %i - SCI_INT32\n", precision);
						break;
					}
					else
					{
						int iRows               = 0;
						int iCols               = 0;
						int* pucData  = NULL;

						/////////////////////
						// 2D INT32 Image //
						/////////////////////
						sciErr = getMatrixOfInteger32(pvApiCtx, piAddr, &iRows, &iCols, &pucData);
						if (sciErr.iErr)
						{
							printError(&sciErr, 0);
							return sciErr.iErr;
						}
						int sz2[] = {iRows,iCols};

						new_img = Mat(2, sz2, CV_32S);
						scidata2matdata(new_img, pucData);
					}

				}
				break;
			case SCI_UINT32:
				//sciErr = getHypermatOfUnsignedInteger32(pvApiCtx, piAddr, &dims, &ndims, (unsigned int*)&data);
				//if(sciErr.iErr)
				//{
				//	printError(&sciErr, 0);
				//	return sciErr.iErr;
				//}
				sciprint("Precision : %i - SCI_UINT32\n", precision);
				break;
			}
		}

		break;

	case sci_pointer:
		sciprint("A pointer\n");
		break;
	default:
		sciprint("Unknown type !\n"); // Should never happen
	}


	/*sciprint("-----------------------\n");
	sciprint("Get Image Properties...\n");
	sciprint("-----------------------\n");
	sciprint("dims : %i\n",new_img.dims);
	sciprint("rows  : %i\n",new_img.rows );
	sciprint("cols  : %i\n",new_img.cols );
	sciprint("channels  : %i\n",new_img.channels() );
	sciprint("dims  : %i\t %i\t %i\t\n",*dims, *(dims+1), *(dims+2));
	sciprint("depth  : %i\n",new_img.depth());*/



	return 0;
}

int GetString(int nPos, char *&pstName,void* pvApiCtx)
{

	SciErr sciErr;
	int* piAddr = NULL;
	int iType   = 0;
	int iRet    = 0;

	sciErr = getVarAddressFromPosition(pvApiCtx, nPos, &piAddr);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return -1;
	}

	if(isStringType(pvApiCtx, piAddr))
	{
		//sciprint("Is String Type\n");
		if(isScalar(pvApiCtx, piAddr))
		{
			char* pstData = NULL;
			//char* pstName = NULL;


			iRet = getAllocatedSingleString(pvApiCtx, piAddr, &pstData);
			//sciprint("Answer (Single String) : %s\n",  pstData);
			pstName = pstData;


			if(iRet)
			{
				freeAllocatedSingleString(pstData);
				return iRet;
			}


		}
		else
		{
			sciprint("Not Scalar Type\n");
			return -1;
		}

		//AssignOutputVariable(pvApiCtx, 1) = nbInputArgument(pvApiCtx) + 1;
	}
	else
	{
		sciprint("Not String Type\n");
		return -1;
		//AssignOutputVariable(pvApiCtx, 1) = 0;
	}
	return 0;
}

//int GetString(int nPos, char *&pstName)
int GetDouble(int nPos, double *&pstdata, int& iRows,int& iCols,void* pvApiCtx)
{

	SciErr sciErr;
	int* piAddr = NULL;
	int iType   = 0;
	int iRet    = 0;


	//CheckInputArgument(pvApiCtx, 1, 1);
	//CheckOutputArgument(pvApiCtx, 0, 1);

	sciErr = getVarAddressFromPosition(pvApiCtx, nPos, &piAddr);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}

	if(isDoubleType(pvApiCtx, piAddr))
	{
		if(isScalar(pvApiCtx, piAddr))
		{
			double dblReal	= 1;
			iRows = 1;
			iCols = 1;
			iRet = getScalarDouble(pvApiCtx, piAddr, &dblReal);
			pstdata = &dblReal;

			if(iRet)
			{
				return iRet;
			}

			//sciprint("%f\n",*pstdata);
		}
		else
		{
			double* pdblReal	= NULL;
			sciErr = getMatrixOfDouble(pvApiCtx, piAddr, &iRows, &iCols, &pdblReal);
			pstdata = pdblReal;

			if(sciErr.iErr)
			{
				printError(&sciErr, 0);
				return sciErr.iErr;
			}


		}

	}
	else 
	{
		printError(&sciErr, 0);
		return -1;
	}

	ReturnArguments(pvApiCtx);
	return 0;
}

// Function from : http://stackoverflow.com/questions/10167534/how-to-find-out-what-type-of-a-mat-object-is-with-mattype-in-opencv

string type2str(int type) {
	string r;

	uchar depth = type & CV_MAT_DEPTH_MASK;
	uchar chans = 1 + (type >> CV_CN_SHIFT);

	switch ( depth ) {
	case CV_8U:  r = "8U"; break;
	case CV_8S:  r = "8S"; break;
	case CV_16U: r = "16U"; break;
	case CV_16S: r = "16S"; break;
	case CV_32S: r = "32S"; break;
	case CV_32F: r = "32F"; break;
	case CV_64F: r = "64F"; break;
	default:     r = "User"; break;
	}

	r += "C";
	r += (chans+'0');

	return r;
}

//int SetImage(int nPos, Mat& new_img)

int is_binary_image( Mat& new_img) {
	int r;

	double s1 = sum(new_img==0)[0]/255;
	double s2 = sum(new_img==255)[0]/255;
	double s3 = new_img.rows*new_img.cols;
	if (s3 == (s1+s2))
		r = 1;
	else
		r = 0; 

	//new_img.convertTo(new_img2,CV_32S,1.0/255);

	return r;
}